#include <csignal>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

const char* s = "!\"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~ ";

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void chargen_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "chargen");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Accepted connection." << std::endl;
      for (int l = 0; ; l = (l+1)%71)
      {
        for (int i = l; i <= l+71; i++)
          conn.put(s[i%95]);
        conn.put('\r');
        conn.put('\n');
      }
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed!" << std::endl;
    }
}

void chargen_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "chargen");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* buf = conn.rdbuf();
      std::cout << "Accepted connection." << std::endl;
      for (int l = 0; ; l = (l+1)%71)
      {
        for (int i = l; i <= l+71; i++)
          buf->sputc(s[i%95]);
        buf->sputc('\r');
        buf->sputc('\n');
      }
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed!" << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: chargen [param]" << std::endl;
      std::cout << "-4\tIPv4 chargen service" << std::endl;
      std::cout << "-6\tIPv6 chargen service" << std::endl;
      std::cout << "-l\tLocal (Unix) chargen service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, signal_interrupt);
    //chargen_socketbuf(argc, argv);
    chargen_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
