#include <iostream>
#include <exception>
#include <socketstream/address.h>
#include <socketstream/iostream.h>

std::iostream& get_connection (int argc, char** argv, char* service)
{
  socketstream::iosocketstream* io;
  socketstream::address*  addr;
  char c = getopt(argc, argv, "4:6:l:");
  if (c == '6')
  {
    io = new socketstream::iosocketstream(socketstream::sockets_base::inet6, socketstream::sockets_base::stream);
    addr = new socketstream::inet6_address(0, optarg);
  }
  else if (c == 'l')
  {
    io = new socketstream::iosocketstream(socketstream::sockets_base::local, socketstream::sockets_base::stream);
    addr = new socketstream::local_address(optarg);
  }
  else
  {
    io = new socketstream::iosocketstream(socketstream::sockets_base::inet, socketstream::sockets_base::stream);
    addr = new socketstream::inet_address(0, optarg);
  }
  io->open(*addr);
  delete addr;
  return *io;
}

int main (int argc, char** argv)
{
 try
 {
    if (argc < 2)
    {
      std::cout << "Usage: client [param]" << std::endl;
      std::cout << "-4 service\tConnect to IPv4 service" << std::endl;
      std::cout << "-6 service\tConnect to IPv6 service" << std::endl;
      std::cout << "-l file\tConnect to Local (Unix) service on file" << std::endl;
      return 0;
    }
  std::iostream& io = get_connection(argc, argv, "echo");
  std::string s;
  while (std::cin >> s)
    io << s << std::flush;
  delete &io;
  return 0;
 }
 catch (const std::exception& e)
 {
  std::cerr << "Exception caught: " << e.what() << std::endl;
  return -1;
 }
}
