#include <csignal>
#include <cstring>
#include <ctime>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void daytime_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "daytime");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Connection accepted." << std::endl;
      time_t t = time(0);
      conn << ctime(&t) << std::flush;
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

void daytime_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "daytime");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* buf = conn.rdbuf();
      std::cout << "Connection accepted." << std::endl;
      time_t t = time(0);
      char* s = ctime(&t);
      buf->sputn(s, std::strlen(s));
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: daytime [param]" << std::endl;
      std::cout << "-4\tIPv4 daytime service" << std::endl;
      std::cout << "-6\tIPv6 daytime service" << std::endl;
      std::cout << "-l\tLocal (Unix) daytime service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, signal_interrupt);
    //daytime_socketbuf(argc, argv);
    daytime_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
