#include <csignal>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void discard_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "discard");
    listener.listen(1);
    char c;
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Connection accepted." << std::endl;
      while ((c = conn.get()) != std::char_traits<char>::eof())
        if (c == '\n')
          std::cout << "\\n" << std::endl;
        else if (c == '\r')
          std::cout << "\\r" << std::endl;
        else
          std::cout << c << std::endl;
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed!" << std::endl;
    }
}

void discard_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "discard");
    listener.listen(1);
    char c;
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* buf = conn.rdbuf();
      std::cout << "Connection accepted." << std::endl;
      while ((c = buf->sbumpc()) != std::char_traits<char>::eof())
        if (c == '\n')
          std::cout << "\\n" << std::endl;
        else if (c == '\r')
          std::cout << "\\r" << std::endl;
        else
          std::cout << c << std::endl;
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: discard [param]" << std::endl;
      std::cout << "-4\tIPv4 discard service" << std::endl;
      std::cout << "-6\tIPv6 discard service" << std::endl;
      std::cout << "-l\tLocal (Unix) discard service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, signal_interrupt);
    //discard_socketbuf(argc, argv);
    discard_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
