#include <csignal>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void
echo_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "echo");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Connection accepted." << std::endl;
      char c;
      while ((c = conn.get()) != std::char_traits<char>::eof())
      {
        if (c == '\n')
        {
          std::cout << "\\n" << std::endl;
          conn.put(c);
          conn.flush();
          continue;
        }
        else if (c == '\r')
          std::cout << "\\r" << std::endl;
        else
          std::cout << c << std::endl;
        conn.put(c);
      }
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

void echo_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "echo");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* buf = conn.rdbuf();
      char c;
      std::cout << "Connection accepted." << std::endl;
      while ((c = buf->sbumpc()) != std::char_traits<char>::eof())
      {
        if (c == '\n')
          std::cout << "\\n" << std::endl;
        else if (c == '\r')
          std::cout << "\\r" << std::endl;
        else
          std::cout << c << std::endl;
        buf->sputc(c);
      }
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: echo [param]" << std::endl;
      std::cout << "-4\tIPv4 echo service" << std::endl;
      std::cout << "-6\tIPv6 echo service" << std::endl;
      std::cout << "-l\tLocal (Unix) echo service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, &signal_interrupt);
    //echo_socketbuf(argc, argv);
    echo_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
