/*
    Copyright (C) 2003 Pedro Lamarao Rosa e Silva
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <irc_msg.h>

void
irc_msg_prefix::parse (const std::string& s)
{
  std::string::size_type i = 0;
  std::string::size_type j = 0;
  std::string::size_type z = s.size();
  if ((i = s.find("!", 0)) != std::string::npos) // parse nick
  {
    _M_nick = s.substr(0, i);
  }
  if ((j = s.find("@", i)) != std::string::npos) // parse user
  {
    _M_user = s.substr(i + 1, j - i - 1);
  }
  _M_host = s.substr(j + 1, z - j - 1); // parse host
  _M_is_empty = false;
}

void
irc_msg::parse (const std::string& s)
{
  this->clear();

  std::string::size_type i = 0;
  std::string::size_type j = 0;
  std::string::size_type z = s.size();

  if (s[z - 1] == '\r') // must be one-before-last, and \r is last
    z--;

  if (s[0] == ':') // parse prefix
  {
    i = s.find(' ', 0);
    _M_prefix.parse(s.substr(1, i - 1));
    i++;
  }

  if ((j = s.find(':', i)) != std::string::npos) // parse text data
  {
    _M_text = s.substr(j + 1, z - j - 1);
    z = j - 1;
  } // move the end (z) to exclude data section

  j = s.find(' ', i); // parse command
  j = (j == std::string::npos ? z : j);
  _M_cmd = s.substr(i, j - i);

  if (j < z) // parse arguments
  {
     const std::string& args = s.substr(j, z - j);
     z = args.size();
     i = 0;
     while ((j = args.find('_', i + 1)) != std::string::npos)
     {
       _M_args.push_back(args.substr(i + 1, j - i - 1));
       i = j;
     }
     _M_args.push_back(args.substr(i + 1, z - i - 1)); // always a leftover
  }
}

std::istream&
operator>> (std::istream& is, irc_msg& ircm)
{
  char buffer[512];
  is.getline(buffer, 512);
  ircm.parse(std::string(buffer));
  return is;
}

std::ostream&
operator<< (std::ostream& os, const irc_msg_prefix& ircmp)
{
  if (ircmp.empty())
    return os;
  if (!ircmp.nick().empty())
    os << ircmp.nick() << "!";
  if (!ircmp.user().empty())
    os << ircmp.user() << "@";
  return os << ircmp.host();
}

std::ostream&
operator<< (std::ostream& os, const irc_msg& ircm)
{
  if (!ircm.prefix().empty())
    os << ":" << ircm.prefix() << " ";
  os << ircm.cmd();
  irc_msg::args_t::const_iterator it = ircm.args().begin();
  while (it != ircm.args().end())
    os << " " << *it++;
  if (!ircm.text().empty())
    os << " :" << ircm.text();
  return os << "\r\n";
}
