#include <csignal>
#include <ctime>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

const char* q = "The Matrix is everywhere. It is all around us.";

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void quote_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "quote");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Connection accepted." << std::endl;
      conn << q << std::endl;
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

void quote_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "quote");
    listener.listen(1);
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* buf = conn.rdbuf();
      std::cout << "Connection accepted." << std::endl;
      buf->sputn(q, 46);
      buf->sputn("\r\n", 2);
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: quote [param]" << std::endl;
      std::cout << "-4\tIPv4 quote service" << std::endl;
      std::cout << "-6\tIPv6 quote service" << std::endl;
      std::cout << "-l\tLocal (Unix) quote service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, signal_interrupt);
    //quote_socketbuf(argc, argv);
    quote_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
