#include <csignal>
#include <ctime>
#include <iostream>
#include <socketstream/iostream.h>
#include "get_listener.h"

#define TIME_OFFSET         2208988800UL

// Call exit() on SIGINT to help on profiling
void
signal_interrupt (int signal)
{
  exit(0);
}

void get_time (char* s)
{
  std::time_t now;
  std::time(&now);
  unsigned long i32 = static_cast<unsigned long>(now) + TIME_OFFSET;
  // assume we really got a string of the appropriate size
  s[0] = i32 >> 24;
  s[1] = i32 >> 16;
  s[2] = i32 >> 8;
  s[3] = i32;
}

void time_iostream (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "time");
    listener.listen(1);
    char buf[3];
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::cout << "Connection accepted." << std::endl;
      get_time(buf);
      conn << buf << std::flush;
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

void time_socketbuf (int argc, char** argv)
{
    socketstream::listener &listener = get_listener(argc, argv, "time");
    listener.listen(1);
    char buf[4];
    while (true)
    {
      socketstream::iosocketstream &conn = listener.accept();
      std::basic_streambuf<char, std::char_traits<char> >* sbuf = conn.rdbuf();
      std::cout << "Connection accepted." << std::endl;
      get_time(buf);
      sbuf->sputn(buf, 3);
      conn.shutdown(socketstream::sockets_base::rw);
      std::cout << "Connection closed." << std::endl;
    }
}

int main (int argc, char** argv)
{
  try
  {
    if (argc < 2)
    {
      std::cout << "Usage: time [param]" << std::endl;
      std::cout << "-4\tIPv4 time service" << std::endl;
      std::cout << "-6\tIPv6 time service" << std::endl;
      std::cout << "-l\tLocal (Unix) time service on ./service.sock" << std::endl;
      return 0;
    }
    std::signal(SIGINT, signal_interrupt);
    //time_socketbuf(argc, argv);
    time_iostream(argc, argv);
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
  return 0;
}
