// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file inet6_factory.h
 * @brief Contains the IPv6 address class.
 */

#ifndef SOCKETSTREAM_INET6_ADDRESS_H
#define SOCKETSTREAM_INET6_ADDRESS_H 1

#include <string>
#include <netdb.h> // for struct addrinfo
#include <socketstream/address_factory.h>

namespace socketstream {

  /**
   * @brief IPv6 address class.
   * 
   * This implementation of the address interface deal with IPv6 addresses
   * and DNS name resolution.
   */
  class inet6_address
  : public address
  {
  public:

    /**
     * @brief Type of the address structure on the system.
     * 
     * This is the structure passed to ::connect(), ::bind(), ::accept(),
     * ::recvfrom() and ::sendto() system calls representing a socket address.
     */
    typedef sockaddr_in6 address_t;

    /**
     * @brief Constructor.
     */
    inet6_address (const char* __node, const char* __serv) throw ();

    /**
     * @brief Constructor
     */
    inet6_address (int __node, const char* __serv) throw ();

    /**
     * @brief Destructor.
     */
    virtual
    ~inet6_address () throw ();

    virtual inet6_address&
    style (sockets_base::style __style);

    virtual inet6_address&
    passive ();

    virtual inet6_address&
    numeric ();

  protected:

    virtual sockaddr*
    resolve () throw (address_exception);

    virtual socklen_t
    size () throw () { return sizeof(address_t); }

  private:

    std::string       _M_node;
    std::string       _M_serv;

    addrinfo*    _M_current;
    addrinfo     _M_hint;

  };
}

#endif // SOCKETSTREAM_INET6_ADDRESS_H
