// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file stream_option.h
 * @brief Contains the stream option class.
 */

#ifndef SOCKETSTREAM_STREAM_OPTION_H
#define SOCKETSTREAM_STREAM_OPTION_H 1

namespace socketstream {
  /**
   * @brief Interface for stream options.
   * 
   * Implementations of this interface know about a certain socket level
   * set of options and their expected value buffers.
   */
  class option {
  public:
    
    /**
     * @brief Destructor.
     */
    virtual
    ~option () { };
    
    /**
     * @brief Returns option level.
     */
    virtual int
    level () = 0;
    
    /**
     * @brief Returns option code.
     */
    virtual int
    code () = 0;
    
    /**
     * @brief Returns option value buffer.
     * 
     * This either contains the value to set the option,
     * or will contain the value to get the option.
     */
    virtual char*
    value () = 0;
    
    /**
     * @brief Returns option value buffer size.
     * 
     * If we are getting an option, instead of setting,
     * this will hold the actual size of stored data.
     */
    virtual size_t
    size () = 0;

  };    
}

#endif // SOCKETSTREAM_STREAM_OPTION_H
