// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file inet_factory.cc
 * @doctodo
 */

#include <socketstream/inet_factory.h>
#include <netdb.h> // for macros

namespace socketstream {

  inet_address::inet_address (const char* __node, const char* __serv) throw ()
  : _M_node(__node), _M_serv(__serv), _M_current(0)
  {
    _M_hint.ai_flags    = 0;
    _M_hint.ai_family   = sockets_base::inet;
    _M_hint.ai_socktype = 0;
    _M_hint.ai_protocol = 0;
  }

  inet_address::inet_address (int __node, const char* __serv) throw ()
  : _M_node(), _M_serv(__serv), _M_current(0)
  {
    _M_hint.ai_flags    = 0;
    _M_hint.ai_family   = sockets_base::inet;
    _M_hint.ai_socktype = 0;
    _M_hint.ai_protocol = 0;
  }

  inet_address::~inet_address () throw ()
  { if (_M_current) freeaddrinfo(_M_current); }

  inet_address&
  inet_address::style(sockets_base::style __style)
  {
    _M_hint.ai_socktype = static_cast<int>(__style);
    return *this;
  }

  inet_address&
  inet_address::passive()
  {
    _M_hint.ai_flags |= AI_PASSIVE;
    return *this;
  }

  inet_address&
  inet_address::numeric()
  {
    _M_hint.ai_flags |= AI_NUMERICHOST;
    return *this;
  }

  sockaddr*
  inet_address::resolve () throw (address_exception)
  {
    const char* __node = _M_node.size() ? _M_node.c_str() : 0;
    const char* __serv = _M_serv.c_str();
    int __r = getaddrinfo(__node, __serv, &_M_hint, &_M_current);
    if (__r)
      throw address_exception(__r);
    return _M_current != 0 ? _M_current->ai_addr : 0;
  }
}
