// -*- Socket Streams -*-

// Copyright (C) 2003 Pedro Lamarao <pedro.lamarao@izcom.com.br>

// This file is part of the Socket Streams Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License
// or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License along
// with this library; see the file COPYING. If not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// USA

/** @file sockets_base.cc
 * @doctodo
 */

#include <socketstream/sockets_base.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>

namespace socketstream {

  sockets_base::family sockets_base::local   = sockets_base::family(PF_LOCAL);
  sockets_base::family sockets_base::inet    = sockets_base::family(PF_INET);
  sockets_base::family sockets_base::inet6   = sockets_base::family(PF_INET6);
#if HAVE_DECL_PF_NETLINK
  sockets_base::family sockets_base::netlink = sockets_base::family(PF_NETLINK);
#else
  sockets_base::family sockets_base::netlink = sockets_base::family(0);
#endif
#if HAVE_DECL_PF_PACKET
  sockets_base::family sockets_base::packet  = sockets_base::family(PF_PACKET);
#else
  sockets_base::family sockets_base::packet = sockets_base::family(0);
#endif

  sockets_base::style sockets_base::stream    = sockets_base::style(SOCK_STREAM);
  sockets_base::style sockets_base::datagram  = sockets_base::style(SOCK_DGRAM);
  sockets_base::style sockets_base::raw       = sockets_base::style(SOCK_RAW);
#if HAVE_DECL_SOCK_RDM
  sockets_base::style sockets_base::rdm       = sockets_base::style(SOCK_RDM);
#else
  sockets_base::style sockets_base::rdm = sockets_base::style(0);
#endif
#if HAVE_DECL_SOCK_SEQPACKET
  sockets_base::style sockets_base::seqpacket = sockets_base::style(SOCK_SEQPACKET);
#else
  sockets_base::style sockets_base::seqpacket = sockets_base::style(0);
#endif
  
  sockets_base::shutmode sockets_base::r  = sockets_base::shutmode(SHUT_RD);
  sockets_base::shutmode sockets_base::w  = sockets_base::shutmode(SHUT_WR);
  sockets_base::shutmode sockets_base::rw = sockets_base::shutmode(SHUT_RDWR);
  
  sockets_base::io_flags sockets_base::oob          = sockets_base::io_flags(MSG_OOB);
  sockets_base::io_flags sockets_base::no_oob       = sockets_base::io_flags(~MSG_OOB);
  sockets_base::io_flags sockets_base::dontroute    = sockets_base::io_flags(MSG_DONTROUTE);
  sockets_base::io_flags sockets_base::no_dontroute = sockets_base::io_flags(~MSG_DONTROUTE);
  sockets_base::io_flags sockets_base::dontwait     = sockets_base::io_flags(MSG_DONTWAIT);
  sockets_base::io_flags sockets_base::no_dontwait  = sockets_base::io_flags(~MSG_DONTWAIT);
#if HAVE_DECL_MSG_NOSIGNAL
  sockets_base::io_flags sockets_base::nosignal     = sockets_base::io_flags(MSG_NOSIGNAL);
  sockets_base::io_flags sockets_base::no_nosignal  = sockets_base::io_flags(~MSG_NOSIGNAL);
#else
  sockets_base::io_flags sockets_base::nosignal     = sockets_base::io_flags(0);
  sockets_base::io_flags sockets_base::no_nosignal  = sockets_base::io_flags(0);
#endif
  sockets_base::io_flags sockets_base::peek         = sockets_base::io_flags(MSG_PEEK);
  sockets_base::io_flags sockets_base::no_peek      = sockets_base::io_flags(~MSG_PEEK);
  sockets_base::io_flags sockets_base::waitall      = sockets_base::io_flags(MSG_WAITALL);
  sockets_base::io_flags sockets_base::no_waitall   = sockets_base::io_flags(~MSG_WAITALL);
  sockets_base::io_flags sockets_base::trunc        = sockets_base::io_flags(MSG_TRUNC);
  sockets_base::io_flags sockets_base::no_trunc     = sockets_base::io_flags(~MSG_TRUNC);
#if HAVE_DECL_MSG_ERRQUEUE
  sockets_base::io_flags sockets_base::errqueue     = sockets_base::io_flags(MSG_ERRQUEUE);
  sockets_base::io_flags sockets_base::no_errqueue  = sockets_base::io_flags(~MSG_ERRQUEUE);
#else
  sockets_base::io_flags sockets_base::errqueue     = sockets_base::io_flags(0);
  sockets_base::io_flags sockets_base::no_errqueue  = sockets_base::io_flags(0);
#endif

  sockets_base::family   sockets_base::def::family   = sockets_base::inet;
  sockets_base::style    sockets_base::def::style    = sockets_base::stream;
  sockets_base::protocol sockets_base::def::protocol = sockets_base::protocol(0);
  sockets_base::io_flags sockets_base::def::io_flags = sockets_base::nosignal;
  
}
