#include <cstring>
#include <iostream>
#include <exception>
#include <socketstream/basic_socket.h>
#include <socketstream/basic_socketbuf.h>
#include <socketstream/inet_factory.h>

const char* msg = "Hello World!";
const int msgsize = 12;
const int bufsize = 24;

// Tests I/O on a basic_socketbuf(const basic_socket&),
// using sputn() for output, sbumpc() for input.

int main (int argc, char** argv)
{
  try
  {
    socketstream::inet_address addr(0, "echo");
    socketstream::basic_socket s;
    s.connect(addr);
    socketstream::basic_socketbuf<char, std::char_traits<char> > sock(s);

    std::cout << "sending: \"" << msg << "\"" << std::endl;
    sock.sputn(msg, msgsize);
    sock.pubsync();

    char buffer[bufsize] = "";
    for (int i = 0; i < msgsize; i++)
      buffer[i] = sock.sbumpc();
    std::cout << "received: \"" << buffer << "\"" << std::endl;

    int ret = memcmp(buffer, msg, msgsize);
    sock.shutdown(socketstream::sockets_base::rw);
    return ret;
  }
  catch (const std::exception& e)
  {
    std::cerr << "Exception caught: " << e.what() << std::endl;
    return -1;
  }
}
